local L = LibStub("AceLocale-3.0"):GetLocale("ShockAndAwe")
local media = LibStub:GetLibrary("LibSharedMedia-3.0")

local format, len, lower = _G.string.format, _G.string.len, _G.string.lower
local gsub, trim = _G.string.gsub, _G.strtrim

ShockAndAwe.ExportFrame = CreateFrame("Frame", "SAA_ExportFrame", UIParent, "DialogBoxFrame")

function ShockAndAwe:CreateExportFrame()
	local frame = ShockAndAwe.ExportFrame
	frame:SetFrameStrata("BACKGROUND")
	frame:SetWidth(400)
	frame:SetHeight(500)
	frame:SetBackdrop({
		bgFile = [[Interface\DialogFrame\UI-DialogBox-Background]],
	    edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
	    tile = true, tileSize = 16, edgeSize = 16,
	    insets = { left = 5, right = 5, top = 5, bottom = 5 }
	})
	frame:SetBackdropColor(0, 0, 0, 1);
	frame:SetMovable(true);
	frame:EnableMouse(true)
	frame:RegisterForDrag("LeftButton");
	frame:SetScript("OnDragStart", 
		function()
			frame:StartMoving();
		end );
	frame:SetScript("OnDragStop",
		function()
			frame:StopMovingOrSizing();
			frame:SetScript("OnUpdate", nil);
		end );
	frame:SetPoint("CENTER")

	if not frame.titleText then
		frame.titleText = frame:CreateFontString(nil, "OVERLAY", "GameFontHighlightLarge")
	end
	frame.titleText:SetPoint("TOP", 0, -5)
	frame.titleText:SetTextColor(1,1,1,1)
	if not frame.outputtext then
		frame.outputtext = frame:CreateFontString(nil, "OVERLAY")
	end
	frame.outputtext:SetTextColor(1,1,1,1)
	frame.outputtext:SetFont("Friz Quadrata TT", 14, "OUTLINE")
	frame.outputtext:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, -5)
	frame.outputtext:SetText("")
	if not frame.line then
		frame.line = frame:CreateFontString(nil, "OVERLAY")
	end
	frame.line:SetFont("Friz Quadrata TT", 14, "OUTLINE");
	frame.maxWidth = 275 -- set to the minimum width we want the frame to be
	frame.lines = 0
	frame.textHeight = frame.line:GetHeight()
end

function ShockAndAwe.ExportFrame:AddLine(text)
	local frame = ShockAndAwe.ExportFrame
	frame.line:SetText(text)
	local lineWidth = frame.line:GetWidth()
	if lineWidth > frame.maxWidth then 
		frame.maxWidth = linewidth
	end
	local outputtext = frame.outputtext:GetText().."\n"..trim(text)
	frame.outputtext:SetText()
	frame.lines = frame.lines + 1
end

function ShockAndAwe:DisplayExportFrame()
	local frame = ShockAndAwe.ExportFrame
	frame:SetWidth(finalWidth)
	frame:SetHeight(frame.lines * (frame.textHeight + 1) + 60)
	frame.line:Hide()  -- hide the temporary holding line
	frame.outputtext:Show()
	frame:Show()
end

function ShockAndAwe:ExportToSim()
	local rating, percentage
	local frame = ShockAndAwe.ExportFrame
	self:CreateExportFrame()
	if CheckIfBuffed() then
		self:PrintMsg(L["warn_sim_export"], self.db.char.warning.colour, 5)
	end
	frame.titleText:SetText("ShockAndAwe Export to EnhSim")
	frame:AddLine("###############################")
	frame:AddLine("## ShockAndAwe EnhSim Export ##")
	frame:AddLine("###############################")
	
	local mh, oh = UnitAttackSpeed("player")
	local haste = GetCombatRatingBonus(18)
	local mhspeed = mh / (1 - haste/100)
	local ohspeed = oh / (1 - haste/100)
	frame:AddLine("mh_speed             "..format("%.2f",mhspeed))
	frame:AddLine("oh_speed             "..format("%.2f",ohspeed))
	
	frame:AddLine("mh_dps              143.5 ## this figure not yet available from addon please enter manually")
	frame:AddLine("oh_dps              143.5 ## this figure not yet available from addon please enter manually")
	
	frame:AddLine("mh_crit             "..format("%.2f",GetCritChance()))
	frame:AddLine("oh_crit             "..format("%.2f",GetCritChance()))
	
	percentage = GetCombatRatingBonus(6) -- Melee Hit rating
	frame:AddLine("mh_hit              "..format("%.2f",percentage))
	frame:AddLine("oh_hit              "..format("%.2f",percentage))
	
	rating = GetCombatRating(24) -- Expertise rating
	frame:AddLine("mh_expertise_rating "..format("%.2f",rating))
	frame:AddLine("oh_expertise_rating "..format("%.2f",rating))
	
	local ap, buff, debuff = UnitAttackPower("player")
	local power = ap + buff + debuff
	frame:AddLine("ap                   "..power)
	frame:AddLine("haste               "..format("%.2f",haste))
	
	percentage = GetCombatRatingBonus(25) -- Armour Penetration rating
	frame:AddLine("armor_penetration   "..format("%.2f",percentage))
	
	frame:AddLine("str                  "..UnitStat("player",1))
	frame:AddLine("agi                  "..UnitStat("player",2))
	frame:AddLine("int                  "..UnitStat("player",4))
	
	frame:AddLine("spellpower        "..GetSpellBonusDamage(4))
	frame:AddLine("spell_crit          "..format("%.2f",GetSpellCritChance(4)))
	
	percentage = GetCombatRatingBonus(8) -- Spell hit rating
	frame:AddLine("spell_hit           "..format("%.2f",percentage))
	
	frame:AddLine("base_mana           4396 ## hardcoded for level 80 level 70 should be 2678") -- base mana hardcoded
	
	if self.db.char.binding.mhspell == L["Windfury Weapon"] then
		frame:AddLine("mh_imbue        windfury")
	elseif self.db.char.binding.mhspell == L["Flametongue Weapon"] then
		frame:AddLine("mh_imbue        flametongue")
	else
		frame:AddLine("mh_imbue        -")
	end
	if self.db.char.binding.ohspell == L["Windfury Weapon"] then
		frame:AddLine("oh_imbue        windfury")
	elseif self.db.char.binding.ohspell == L["Flametongue Weapon"] then
		frame:AddLine("oh_imbue        flametongue")
	else
		frame:AddLine("oh_imbue        -")
	end
	frame:AddLine("")
	self.ExportItemDetails("mh_enchant", 16, true)
	self.ExportItemDetails("oh_enchant", 17, true)
	frame:AddLine("")
	self.ExportItemDetails("trinket1", 13, false)
	self.ExportItemDetails("trinket2", 14, false)
	frame:AddLine("")
	self.ExportItemDetails("totem", 18, false)
	frame:AddLine("")
	frame:AddLine("")
	self.ExportTalents()
	self:DisplayExportFrame()
end

function ShockAndAwe:ExportItemDetails(slotname, slotid, enchant)
	local frame = ShockAndAwe.ExportFrame
	local slotLink = GetInventoryItemLink("player", slotid)
	if slotLink then
		local itemName, itemString = GetItemInfo(slotLink)
		if self.db.char.debug then
			local printable = gsub(itemString, "\124", "\124\124");
			self:Print(slotName.." contains: \"" .. printable .. "\"");
		end
		if enchant then -- looking up weapon so we want enchant details
			local _, _, enchantID = strsplit(":", itemString)
			frame:AddLine(slotname.."    "..getEnchantName(enchantID))
		else
			frame:AddLine(slotname.."    "..formatItemName(itemName))
		end
	else
		frame:AddLine(slotname.."    -")
	end
end

local function formatItemName(name)
	if not name then 
		return "-"
	end
	local outname = gsub(name, "_", " ")
	return lower(outname)
end

local function getEnchantName(id)
	if not id then
		return "-"
	end
	if id == 2673 then
		return "mongoose"
	elseif id == 3225 then
		return "executioner"
	elseif id == 1900 then
		return "crusader"
	elseif id == 3273 then
		return "deathfrost"
	else
		return "-"
	end
end

function ShockAndAwe:CheckIfBuffed()
	local index = 1
	while UnitBuff("PLAYER", index) do
		local name = UnitBuff("PLAYER", index)
		if name ~= L["Heroic Presence"] then
			index = index + 1
		end
	end
	if index > 0 then 
		return true
	else
		return false
	end
end

function ShockAndAwe:ExportTalents()
	local frame = ShockAndAwe.ExportFrame
	local currRank, maxRank
	frame:AddLine("#############")
	frame:AddLine("## Talents ##")
	frame:AddLine("#############")

	--nameTalent, icon, tier, column, currRank, maxRank= GetTalentInfo(tree,index)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,3) -- Ancestral Knowledge
	frame:AddLine("ancestral_knowledge       "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,7) -- Improved Shields
	frame:AddLine("improved_shields          "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,15) -- Mental Dexterity
	frame:AddLine("mental_dexterity          "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,9) -- Shamanistic Focus
	frame:AddLine("shamanistic_focus         "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,11) -- Flurry
	frame:AddLine("flurry                    "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,8) -- Elemental Weapons
	frame:AddLine("elemental_weapons         "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,16) -- Unleashed Rage
	frame:AddLine("unleashed_rage            "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,17) -- Weapon Mastery
	frame:AddLine("weapon_mastery            "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,18) -- Dual Wield Specialisation
	frame:AddLine("dual_wield_specialization "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,21) -- Mental Quickness
	frame:AddLine("mental_quickness          "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,23) -- Imp.Stormstrike
	frame:AddLine("improved_stormstrike      "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,24) -- Static Shock
	frame:AddLine("static_shock              "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(2,27) -- Maelstrom Weapon
	frame:AddLine("maelstrom_weapon          "..currRank.."/"..maxRank)
	
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,1) -- Convection
	frame:AddLine("convection                "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,2) -- Concussion
	frame:AddLine("concussion                "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,3) -- Call of Flame
	frame:AddLine("call_of_flame             "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,5) -- Elemental Devastation
	frame:AddLine("elemental_devastation     "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,6) -- Reverberation
	frame:AddLine("reverberation             "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,7) -- Elemental Focus
	frame:AddLine("elemental_focus           "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,8) -- Elemental Fury
	frame:AddLine("elemental_fury            "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,12) -- Call of Thunder
	frame:AddLine("call_of_thunder           "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,14) -- Elemental Precision
	frame:AddLine("elemental_precision       "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,15) -- Lightning Mastery
	frame:AddLine("lightning_mastery         "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,18) -- Elemental Oath
	frame:AddLine("elemental_oath            "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,19) -- Lightning Overload
	frame:AddLine("lightning_overload        "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,22) -- Lava Flows
	frame:AddLine("lava_flows                "..currRank.."/"..maxRank)
	_, _, _, _, currRank, maxRank = GetTalentInfo(1,23) -- Storm Earth and Fire
	frame:AddLine("storm_earth_and_fire      "..currRank.."/"..maxRank)
end