--- Many thanks to Dennis Hafstrom for the idea (and the code) for this News frame (lifted from Enhancer)

local _G = getfenv(0)
local LibStub = _G["LibStub"]
local L = LibStub("AceLocale-3.0"):GetLocale("ShockAndAwe")

local format, len = _G.string.format, _G.string.len
local gsub, trim = _G.string.gsub, _G.strtrim

ShockAndAwe.newsCurrent = 1 -- Can only ever increase

function ShockAndAwe:AddNewsStory(article)
	if article == 1 then
		ShockAndAwe.newsFrame:AddLine("Introduction", "Welcome to ShockAndAwe. This first message is only a test of the News frame.")
		ShockAndAwe.newsFrame:AddLine("", "There are lots of config options you can use in ShockAndAwe, please use the")
		ShockAndAwe.newsFrame:AddLine("", "Blizzard config options for easy configuration.")
		ShockAndAwe.newsFrame:AddLine("", "Please also report any bugs you find to the website below. However before you")
		ShockAndAwe.newsFrame:AddLine("", "post a bug report PLEASE do a /saa version and quote the full version number.")
		ShockAndAwe.newsFrame:AddLine("", "When you post a bug. A report without a version number is practically useless.")
	end
end

function ShockAndAwe:News(override)
	if (not override and ((self.db.char.newsitem or 0) >= self.newsCurrent)) then return; end
	self:CreateNewsFrame()
	
	ShockAndAwe.newsFrame:Clear()
	ShockAndAwe.newsFrame.title:SetText("|cffffd200ShockAndAwe|r")
	
	self:AddNewsStory(self.newsCurrent)
	
	-------- max length for "subject    ","body"---------------------------------------------------
	ShockAndAwe.newsFrame:AddLine("            ","") -- Some spacing
	ShockAndAwe.newsFrame:AddLine("            ","")
	
	ShockAndAwe.newsFrame:AddLine(L["Config"], L["help1"])
	ShockAndAwe.newsFrame:AddLine("", L["help2"])
	ShockAndAwe.newsFrame:AddLine(L["Command"], L["help_command"])
	ShockAndAwe.newsFrame:AddLine(L["Website"], self:SpamSafe(L["__URL__"]))
	
	-- do return; end
	ShockAndAwe.newsFrame:Show()
	self.db.char.newsitem = self.newsCurrent
end

function ShockAndAwe:SpamSafe(text)
	-- replace ' [dot] ' with '.' and ' [at] ' with '@'
	return gsub(gsub(text, " %[at%] ", "@"), " %[dot%] ", ".")
end

function ShockAndAwe:CreateNewsFrame()
	ShockAndAwe.newsFrame = CreateFrame("Frame", "AddOnNewsFrame", UIParent, "DialogBoxFrame")
	ShockAndAwe.newsFrame:SetWidth(500)
	ShockAndAwe.newsFrame:SetHeight(400)
	ShockAndAwe.newsFrame:SetPoint("CENTER")
	ShockAndAwe.newsFrame:SetBackdrop({
		bgFile = [[Interface\DialogFrame\UI-DialogBox-Background]],
	    edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
	    tile = true, tileSize = 16, edgeSize = 16,
	    insets = { left = 5, right = 5, top = 5, bottom = 5 }
	})
	ShockAndAwe.newsFrame:SetBackdropColor(0,0,0,1)

	local text = ShockAndAwe.newsFrame:CreateFontString(nil, "OVERLAY", "GameFontHighlightLarge")
	text:SetFont([[Interface\AddOns\ShockAndAwe\fonts\CAS_ANTN.TTF]], 24, "OUTLINE") -- http://wdnaddons.com/2307561/FrameXML/Fonts.xml
	ShockAndAwe.newsFrame.title = text
	text:SetPoint("TOP", 0, -5)

	ShockAndAwe.newsFrame:Hide()

	ShockAndAwe.newsFrame.lefts = {}
	ShockAndAwe.newsFrame.rights = {}
	ShockAndAwe.newsFrame.textLefts = {}
	ShockAndAwe.newsFrame.textRights = {}
	function ShockAndAwe.newsFrame:Clear()
		self.title:SetText("")
		self.disclaimer:SetText("")
		for i = 1, #self.lefts do
			self.lefts[i] = nil
			self.rights[i] = nil
		end
	end
	
	local function rightColor(right)
		-- Email color as link
		right = gsub(right, "[A-Za-z0-9%.%%%+%-]+@[A-Za-z0-9%.%%%+%-]+%.%w%w%w?%w?", function(v) return "|cff007fff"..v.."|r"; end)
		
		-- URL color as link (is crap need better URL matching)
		right = gsub(right, "http%:%/%/[A-Za-z0-9%.%/%-]+", function(v) return "|cff007fff"..v.."|r"; end)
		
		-- '-quoted color gray
		right = gsub(right, "(%')(%/%w-%s?%w-)(%')", function(v1, v2, v3) return v1.."|cffcccccc"..v2.."|r"..v3; end) --00ff7f
		
		-- *) lists
		right = gsub(right, "^(%*%)%s)(.+)", function(v1, v2) return "|cff00cccc"..v1.."|r"..v2; end)
		
		-- return the mess
		return right
	end

	function ShockAndAwe.newsFrame:AddLine(left, right)
		ShockAndAwe.newsFrame.lefts[#ShockAndAwe.newsFrame.lefts+1] = trim(left)
		ShockAndAwe.newsFrame.rights[#ShockAndAwe.newsFrame.rights+1] = rightColor(trim(right))
	end

	local newsFrame_Show = ShockAndAwe.newsFrame.Show
	function ShockAndAwe.newsFrame:Show(...)
		local maxLeftWidth = 0
		local maxRightWidth = 0
		local textHeight = 0
		for i = 1, #self.lefts do
			if not self.textLefts[i] then
				local left = ShockAndAwe.newsFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
				left:SetFont([[Interface\AddOns\ShockAndAwe\fonts\CAS_ANTN.TTF]], 16, "OUTLINE")
				self.textLefts[i] = left
				local right = ShockAndAwe.newsFrame:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
				right:SetFont([[Interface\AddOns\ShockAndAwe\fonts\Cella.ttf]], 14, "OUTLINE")
				self.textRights[i] = right
				if i == 1 then
					left:SetPoint("TOPRIGHT", ShockAndAwe.newsFrame, "TOPLEFT", 100, -35)
				else
					left:SetPoint("TOPRIGHT", self.textLefts[i-1], "BOTTOMRIGHT", 0, -1)
				end
				right:SetPoint("LEFT", left, "RIGHT", 5, 0)
			end
			self.textLefts[i]:SetText(self.lefts[i] .. ((len(self.lefts[i]) > 0 and len(strtrim(self.lefts[i])) > 0 and ":") or " "))
			self.textRights[i]:SetText(self.rights[i])
			local leftWidth = self.textLefts[i]:GetWidth()
			local rightWidth = self.textRights[i]:GetWidth()
			textHeight = self.textLefts[i]:GetHeight()
			if maxLeftWidth < leftWidth then
				maxLeftWidth = leftWidth
			end
			if maxRightWidth < rightWidth then
				maxRightWidth = rightWidth
			end
		end
		for i = #self.lefts+1, #self.textLefts do
			self.textLefts[i]:SetText('')
			self.textRights[i]:SetText('')
		end
		local finalWidth = maxLeftWidth + maxRightWidth + 10
		if (finalWidth < 360) then finalWidth = 360; end
		ShockAndAwe.newsFrame:SetWidth(finalWidth)
		ShockAndAwe.newsFrame:SetHeight(#self.lefts * (textHeight + 1) + 100)

		newsFrame_Show(self, ...)
	end
	ShockAndAwe.newsFrame:Hide()
	
	local disclaimer = ShockAndAwe.newsFrame:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
	disclaimer:SetFont([[Interface\AddOns\ShockAndAwe\fonts\Cella.ttf]], 8, "")
	ShockAndAwe.newsFrame.disclaimer = disclaimer
	disclaimer:SetPoint("BOTTOMRIGHT", ShockAndAwe.newsFrame:GetName(), "BOTTOMRIGHT", -5, 5)
	
	ShockAndAwe.CreateNewsFrame = function() return; end;
end